#!/usr/bin/env node
"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var ExpressBootstrap_1 = require("../lib/ExpressBootStrap/ExpressBootstrap");
var express_1 = __importDefault(require("express"));
var command_line_args_1 = __importDefault(require("command-line-args"));
var cli = command_line_args_1.default([
    {
        name: "dir",
        alias: "d",
        type: String,
        defaultValue: ".",
    },
    {
        name: "tsc",
        alias: "t",
        type: Boolean,
        defaultValue: false,
    },
    {
        name: "verbose",
        alias: "v",
        type: Boolean,
        defaultValue: false
    },
    {
        name: "cors",
        alias: "c",
        type: Boolean,
        defaultValue: false
    },
    {
        name: "live",
        alias: "l",
        type: Boolean,
        defaultValue: false
    },
    {
        name: "route",
        alias: "r",
        type: String,
        defaultValue: "/",
    },
    {
        name: "ext",
        alias: "x",
        type: String,
        defaultValue: [".ts", ".js"],
        lazyMultiple: true
    },
    {
        name: "port",
        alias: "p",
        type: Number,
        defaultValue: 3000
    },
    {
        name: "help",
        alias: "h",
        type: Boolean,
        defaultValue: false
    }
], {});
if (cli.help) {
    var commandLineUsage = require('command-line-usage');
    var sections = [
        {
            header: 'File Express',
            content: 'Create an api from the file system'
        },
        {
            header: 'Options',
            optionList: [
                {
                    name: 'dir',
                    typeLabel: '{underline Directory}',
                    description: 'The base directory to bootstrap api.',
                    alias: "d"
                },
                {
                    name: "live",
                    typeLabel: '{underline Boolean}',
                    description: "Enable hot reloading of function",
                    alias: "d"
                },
                {
                    name: "cors",
                    typeLabel: '{underline Boolean}',
                    description: "Enable cors for all routes",
                    alias: "c",
                },
                {
                    name: 'ext',
                    typeLabel: '{underline Extension}',
                    description: 'The extension for the files to use. This requires the {underline .} in the extension name',
                    alias: "x"
                },
                {
                    name: 'route',
                    typeLabel: '{underline String}',
                    description: 'A path to place in front of all routes, eg {underline /api/v1}',
                    alias: "r"
                },
                {
                    name: 'port',
                    typeLabel: '{underline Number}',
                    description: 'The port to boot the application on. Default is {underline 3000}',
                    alias: "p"
                },
                {
                    name: 'help',
                    description: 'Print this usage guide.'
                }
            ]
        }
    ];
    var usage = commandLineUsage(sections);
    console.log(usage);
    process.exit(0);
}
if (cli.tsc) {
    require("ts-node/register");
}
var app = express_1.default();
var router = express_1.default.Router();
if (cli.cors) {
    var cors = require("cors");
    router.use(cors());
}
app.use(cli.route, router);
var boot = new ExpressBootstrap_1.ExpressBootstrap(router);
var options = {
    basedir: cli.dir,
    ext: cli.ext,
    live: cli.live,
    verbose: cli.verbose
};
boot.boot(options);
var server = app.listen(process.env.PORT || cli.port, function () {
    var address = server.address();
    console.log("[" + options.basedir + "]:" + address.port);
});
